#!/usr/bin/perl -w

package acidrip;
require 5.005;
use strict;
use Data::Dumper;

use AcidRip::acidrip;

sub quit_acidrip
{
	if ( $::settings->{'mencoder_pid'} > 0 ) {
		my $m = $::settings->{'mencoder_pid'} + 1;
	  kill 'INT', $::settings->{'mencoder_pid'};
    kill 'INT', $m if `ps $m | grep mencoder` || `ps $m | grep mplayer`;
  }
	Gtk2->main_quit;
}

sub on_select_audio
{
	my ($widget, $this_audio) = @_;
	$::settings->{'selected_audio'} = $this_audio;
	if ($this_audio > 0 ) {
		my $audio = get_audio($this_audio);
		message("selected_audio changed to $this_audio - Channels $audio->{'channels'} frequency $audio->{'frequency'} quant. $audio->{'quantization'}");
		set_bitrate();
	}
}

sub on_select_subp
{
	my ($widget, $this_subp) = @_;
	$::settings->{'selected_subp'} = $this_subp;
	message("selected_subp changed to $::settings->{'selected_subp'}");
}

sub on_select_track
{
	my ($selection, $model) = @_;
	my $x;
	my @newitem = $selection->get_selected_rows;
	
	return if not defined $newitem[0];
	
	my $newitem = ($newitem[0]->get_indices)[0] + 1;
	my $this_track = get_track($newitem);
	
	# count total and collapse if odd?
	$::settings->{'selected_chapters_start'} = 0;
	$::settings->{'selected_chapters_end'} = 0;
	foreach (@newitem) {
		($selection->get_tree_view)->collapse_all if ($_->get_depth == 1);
		if ($_->get_depth == 2) {
			$::settings->{'selected_chapters_start'} = ($_->get_indices)[1] + 1 if $::settings->{'selected_chapters_start'} == 0; 
			$::settings->{'selected_chapters_end'} = ($_->get_indices)[1] + 1;
		}
	}
	$::settings->{'selected_track'} = $this_track->{'ix'};
	if ($::settings->{'selected_chapters_start'}) {
  	$::widgets->{'selected_track_label'}->set_text("Selected Track $::settings->{'selected_track'} ".
  		"Ch $::settings->{'selected_chapters_start'}-$::settings->{'selected_chapters_end'} - ".
  		hhmmss(get_selection_length($::settings->{'selected_chapters_start'}, $::settings->{'selected_chapters_end'})))
	} else {
		$::widgets->{'selected_track_label'}->set_text("Selected Track $newitem - " . hhmmss($this_track->{'length'}))
	}

	#load the correct audio entries for the menu
	my $audio_menu = Gtk2::Menu->new;
	$::widgets->{'audio_track_option'}->set_menu($audio_menu);
	my $audio_item = Gtk2::MenuItem->new("<Default> $::settings->{'language'}");
	my $default_audio = -1;
	foreach (reverse(@{$this_track->{'audio'}})) {
		$default_audio = $_->{'ix'} if $_->{'langcode'} eq $::settings->{'language'} || $_->{'language'} eq $::settings->{'language'};
	}
  $audio_item->signal_connect('activate', \&on_select_audio, $default_audio);
	$audio_menu->append($audio_item);
	$audio_item->show;
  $::widgets->{'audio_track_option'}->set_history(0);

	$audio_item = Gtk2::MenuItem->new("<None>");
	$audio_item->signal_connect('activate', \&on_select_audio, -2);
	$audio_menu->append($audio_item);
	$audio_item->show;

	foreach my $this_audio (@{$this_track->{'audio'}})	{
		my $audio_ix = $this_audio->{'ix'};
		my $label = $audio_ix . ": " . $this_audio->{'language'};
		if ( $this_audio->{'content'} ne "Undefined" ) {
			$label = $label . ": " . $this_audio->{'content'}
		}
		$audio_item= Gtk2::MenuItem->new($label);
	  $audio_item->signal_connect('activate', \&on_select_audio, $audio_ix);
		$audio_menu->append($audio_item);
		$audio_item->show;
	}

	my $subp_menu = Gtk2::Menu->new;
	$::widgets->{'subp_option'}->set_menu($subp_menu);
	my $subp_item = Gtk2::MenuItem->new("<None>");
	$subp_item->signal_connect('select', \&on_select_subp, -1);
	$subp_menu->append($subp_item);
	$subp_item->show;
  $::widgets->{'subp_option'}->set_history(0);

	foreach my $this_subp (@{$this_track->{'subp'}})	{
		my $subp_ix = $this_subp->{'ix'};
	  my $label = $subp_ix . ": " . $this_subp->{'language'};
		$label .= ": " . $this_subp->{'content'} if $this_subp->{'content'} ne "Undefined";
		$subp_item = new_with_label Gtk2::MenuItem($label);
	  $subp_item->signal_connect('activate', \&on_select_subp, $subp_ix);
		$subp_menu->append($subp_item);
		$subp_item->show;
	}
#	split_track() if $::settings->{'total_blocks'} > 1;
	set_bitrate();
}

sub on_video_bitrate_lock_check_clicked
{
	$::widgets->{'video_bitrate_entry'}->set_editable(! shift->get_active);
}

sub on_scale_lock_check_clicked
{
	my $widget = shift;
	$::widgets->{'scale_height_spin'}->set_sensitive(! $widget->get_active);
	if ($widget->get_active) {
		$::widgets->{'scale_height_spin'}->hide;
		$::widgets->{'scale_height_estimate_entry'}->show;
		$::widgets->{'scale_height_spin'}->set_value("-2");
	} else {
		$::widgets->{'scale_height_spin'}->show;
		$::widgets->{'scale_height_estimate_entry'}->hide;
		$::widgets->{'scale_height_spin'}->set_value($::widgets->{'scale_height_estimate_entry'}->get_text);
	}
}

sub on_filesize_changed
{
	my $free = 0;
	my $size = $::settings->{'filesize'} * $::settings->{'total_blocks'};
	open(DF, "df $::settings->{'directory'} |") if -w $::settings->{'directory'};
	return if not -w  $::settings->{'directory'};
	while( <DF> ) {
		my @array = split /\s+/;
		$free = $array[3] / 1024 if $. != 1;
	} close DF;
	$free = sprintf ("%d", $free);
	set_warning_text($::widgets->{'filesize_spin'}, $free <= $size ? 1 : 0);
	my $warn_block = $::settings->{'total_blocks'} > 1 ? 1 : 0;
	set_warning_text($::widgets->{'total_blocks_spin'}, $free <= $size ? $warn_block : 0);
	$::settings->{'enough_space'} = $free <= $size ? 0 : 1;
}

sub on_bpp_changed
{
	my $widget = shift;
	my $bpp = $widget->get_text || 0;
	my $message = "Bits per pixel is "; 
	my $warn;
	
	if		($bpp < 0.10) { $message .= "FAR too low!!, increase file size or reduce resolution";	$warn = 1 }
	elsif ($bpp < 0.15) {	$message .= "quite low, output will probably suck, increase filesize or lower resolution";	$warn = 0.5}
	elsif ($bpp < 0.20) {	$message .= "alright, output will be a bit blocky, but generally ok."; $warn = 0}
	elsif ($bpp < 0.25) {	$message .= "super, output should look pretty good"; $warn = 0}
	elsif ($bpp < 0.30) {	$message .= "quite high, not much to gain here. might want a slightly higher resolution"; $warn = 0.5}
	else 								{ $message .= "very high, you'll waste the space, try a bigger resolution"; $warn = 1}
		
	set_warning_text($widget, $warn);
	
	$::widgets->{'tooltips'}->set_tip($widget, $message);
}

sub on_crop_detect_button_clicked
{	
	message("Crop detection in progress... please wait");

	while (Gtk2->events_pending()) {Gtk2->main_iteration()};

	my $crop="fail"; my $crop_output;
	($crop, $crop_output) = find_crop($::settings->{'selected_track'});
	$::widgets->{'mencoder_output_text'}->get_buffer->insert_with_tags_by_name(
		$::widgets->{'mencoder_output_text'}->get_buffer->get_end_iter, $crop_output, 'mplayer');
	
	if (defined $crop && $crop=~/crop=(\d+):(\d+):(\d+):(\d+)/)
	{
  	set_setting('crop_width', $1);
		set_setting('crop_height', $2);
		set_setting('crop_horizontal', $3);
		set_setting('crop_vertical', $4);
		message('Crop detection completed');
	} else { message('Crop detection failed, is the DVD loaded?', 'error') }
}

sub on_crop_preview_button_clicked
{	
	if (defined $::dvd->{'track'})
	{
		$::widgets->{'stop_preview_button'}->show;
		$::widgets->{'preview_button'}->hide;
		$::widgets->{'preview_socket'}->show;

		#fork proofing
		open STDIN,  '/dev/null' or message("Can't read /dev/null: $!. May fail if in background");
		
		message("Preview command: ");
		message("Previewing crop window");
	  while (Gtk2->events_pending()) {Gtk2->main_iteration()};
   	
		my $mplayer_output = '';
		
		my $preview = get_command("preview"); 
		
		message("Preview command: $preview");
		message("Previewing crop window");
			
		$/="\r";
		$::settings->{'mencoder_pid'} = open (PREVIEW, "$preview 2>&1 |");
    while(<PREVIEW>) {
			$::widgets->{'preview_socket'}->set_size_request($1, $2) if /\s=>\s(\d+)x(\d+)/ && $::settings->{'embed_preview'};
			$mplayer_output .= $_ if $_!~/crop|^A:/;
			while (Gtk2->events_pending()) {Gtk2->main_iteration()}
		}
		$/="\n";
		close PREVIEW;
	  message($mplayer_output);
		message("Preview finished");
		$::widgets->{'stop_preview_button'}->hide;
		$::widgets->{'preview_button'}->show;
		$::widgets->{'preview_socket'}->hide;
	} else { message("Preview failed, is the DVD loaded?") }
}

sub on_info_field_select 
{ 
	my ($widget, $value) = @_;
	$::widgets->{'info_name_entry'}->hide;
	$::widgets->{'info_artist_entry'}->hide;
	$::widgets->{'info_genre_entry'}->hide;
	$::widgets->{'info_subject_entry'}->hide;
	$::widgets->{'info_comment_entry'}->hide;
	$::widgets->{'info_copyright_entry'}->hide;
	$::widgets->{'info_' . $value . '_entry'}->show;
}

sub on_setting_finished
{
	my $widget = shift;
	my $name = $::widgets->get_widget_name($widget);
	$name=~/([\w_]+)_\w+/;
	my $data=$1;
	my $ref = ref $widget;
	my $value;
	
	if ($ref eq "Gtk2::Entry") {$value = $widget->get_text}
	elsif ($ref eq "Gtk2::SpinButton") {$value = $widget->get_value_as_int}
	elsif ($ref eq "Gtk2::CheckButton" || $ref eq "Gtk2::ToggleButton") {if ($widget->get_active) {$value = 1} else {$value = 0}}
	$::settings->{$data} = $value;

	message("$data has been set to $value");
	on_setting_changed($widget);
	
	return 0;
}

sub enable_crop_control {
	my $value = shift;
	$::widgets->{'crop_width_spin'}->set_sensitive($value);
	$::widgets->{'crop_width_label'}->set_sensitive($value);
	$::widgets->{'crop_height_spin'}->set_sensitive($value);
	$::widgets->{'crop_height_label'}->set_sensitive($value);
	$::widgets->{'crop_vertical_spin'}->set_sensitive($value);
	$::widgets->{'crop_vertical_label'}->set_sensitive($value);
	$::widgets->{'crop_horizontal_spin'}->set_sensitive($value);
	$::widgets->{'crop_horizontal_label'}->set_sensitive($value);
	$::widgets->{'crop_detect_button'}->set_sensitive($value);
	$::settings->{'crop_width'} = $value ? $::widgets->{'crop_width_spin'}->get_value : 0;
	$::settings->{'crop_height'} = $value ? $::widgets->{'crop_height_spin'}->get_value : 0;
	set_bpp();
}

sub enable_scale_control {
	my $value = shift;
	$::widgets->{'scale_width_spin'}->set_sensitive($value);
	$::widgets->{'scale_width_label'}->set_sensitive($value);
	$::widgets->{'scale_height_spin'}->set_sensitive($value);
	$::widgets->{'scale_height_label'}->set_sensitive($value);
	$::widgets->{'scale_height_estimate_entry'}->set_sensitive($value);
	$::widgets->{'scale_lock_check'}->set_sensitive($value);
	$::settings->{'scale_width'} = $value ? $::widgets->{'scale_width_spin'}->get_value : 0;
	$::settings->{'scale_height'} = $value ? $::widgets->{'scale_height_spin'}->get_value : 0;
	set_bpp();
}

sub on_option_changed
{
	my $widget = shift;
	my $value = shift;
	my $menu = $widget->get_parent;
	my $name = $::widgets->get_widget_name($menu);
	$name =~ /([\w_]+)_\w+/;
	$name = $1;
	$::settings->{$name} = $value;
	message("$1 has been set to $value");

	if ($name eq "video_codec") {
		message($::settings->{'available_video'}{$value});
		my $enable = ($value eq "lavc" || $value eq "xvid" || $value eq "divx4");
		$::widgets->{'video_options_entry'}->set_text(defined $::settings->{$value . '_options'} ? $::settings->{$value . '_options'} : '');
		$::widgets->{'video_bpp_entry'}->set_sensitive($enable);
		$::widgets->{'video_bpp_label'}->set_sensitive($enable);
		$::widgets->{'video_2pass_check'}->set_sensitive($enable);
		$::widgets->{'crop_enable_check'}->set_sensitive($enable);
		$::widgets->{'crop_detect_button'}->set_sensitive($enable);
		$::widgets->{'scale_enable_check'}->set_sensitive($enable);
		$::widgets->{'video_options_entry'}->set_sensitive($enable);
		$::widgets->{'video_bitrate_entry'}->set_sensitive($enable);
		$::widgets->{'video_options_label'}->set_sensitive($enable);
		$::widgets->{'video_bitrate_label'}->set_sensitive($enable);
		$::widgets->{'vf_pre_enable_check'}->set_sensitive($enable);
		$::widgets->{'vf_post_enable_check'}->set_sensitive($enable);
		$::widgets->{'vf_pre_entry'}->set_sensitive($enable);
		$::widgets->{'vf_post_entry'}->set_sensitive($enable);
		enable_crop_control($enable);
		enable_scale_control($enable);
	}
	if ($name eq "audio_codec") {
		message($::settings->{'available_audio'}{$value});
		my $enable = ($value eq "mp3lame");
		$::widgets->{'audio_options_entry'}->set_sensitive($enable);
		$::widgets->{'audio_options_label'}->set_sensitive($enable);
	}
	set_bpp();
}

# Sets trivial data, and UI manipulation
sub on_setting_changed
{
	my $widget = shift;
	my $name = $::widgets->get_widget_name($widget);
	$name=~/([\w_]+)_\w+/;
	my $data=$1;
	my $ref = ref $widget;
	my $value;
	my $boolean;
	my $encode = 1;

	if ($ref eq "Gtk2::Entry") {$value = $widget->get_text}
	elsif ($ref eq "Gtk2::SpinButton") {$value = $widget->get_value_as_int}
	elsif ($ref eq "Gtk2::CheckButton" || $ref eq "Gtk2::ToggleButton") {if ($widget->get_active) {$value = 1} else {$value = 0}}
	$::settings->{$data} = $value;
	
	if ($widget eq $::widgets->{'filesize_spin'}) { 
		set_bitrate() if defined $::dvd->{'track'};
	}
	if ($widget eq $::widgets->{'total_blocks_spin'}) { 
		on_filesize_changed();
		split_track();
		set_bitrate() if defined $::dvd->{'track'};
	}
	if ($widget eq $::widgets->{'tooltips_check'}) {
		if ($widget->get_active) {
			$::widgets->{'tooltips'}->enable
		} else {
			$::widgets->{'tooltips'}->disable
		}
	}
	if ($widget eq $::widgets->{'crop_enable_check'}) {
		enable_crop_control($::widgets->{'crop_enable_check'}->get_active && $encode)
	}
	if ($widget eq $::widgets->{'scale_enable_check'}) {
		enable_scale_control($::widgets->{'scale_enable_check'}->get_active && $encode)
	}
	if ($widget eq $::widgets->{'vf_pre_enable_check'}) {
		$boolean = $::widgets->{'vf_pre_enable_check'}->get_active && $encode;
		$::widgets->{'vf_pre_entry'}->set_sensitive($boolean);
	}
	if ($widget eq $::widgets->{'vf_post_enable_check'}) {
		$boolean = $::widgets->{'vf_post_enable_check'}->get_active && $encode;
		$::widgets->{'vf_post_entry'}->set_sensitive($boolean);
	}
	if ($widget eq $::widgets->{'video_options_entry'}) {
		warn $::settings->{'video_codec'} . "_options";
		$::settings->{$::settings->{'video_codec'} . "_options"} = $::widgets->{'video_options_entry'}->get_text;
	}
	if ($widget eq $::widgets->{'audio_options_entry'}) {
		$::settings->{$::settings->{'audio_codec'} . "_options"} = $::widgets->{'audio_options_entry'}->get_text;
		set_bitrate();
	}
	if ($widget eq $::widgets->{'scale_width_spin'} || $widget eq $::widgets->{'scale_height_spin'} ||
	  $widget eq $::widgets->{'crop_width_spin'} || $widget eq $::widgets->{'crop_height_spin'} ||
		$widget eq $::widgets->{'crop_horizontal_spin'} || $widget eq $::widgets->{'crop_vertical_spin'}) { 
		set_bpp(); 
	}
	if ($widget eq $::widgets->{'directory_filename_entry'}) {
		my @filearray = split/\//, $value;		
		my $fullname = $value . $::settings->{'filetype'};
		$::widgets->{'view_button'}->set_sensitive(-e $fullname ? 1 : 0);
		$::widgets->{'filename_entry'}->set_text($filearray[-1]);
		$value =~ s/$filearray[-1]//;
		$::widgets->{'directory_entry'}->set_text($value);
		on_filesize_changed();	
	}
	if ($widget eq $::widgets->{'filename_entry'} || $widget eq $::widgets->{'directory_entry'}) {
		my $fullname = $::settings->{'directory'} . $::settings->{'filename'} . $::settings->{'filetype'};
		$::widgets->{'view_button'}->set_sensitive(-e $fullname ? 1 : 0);
		$::widgets->{'directory_filename_entry'}->set_text($::settings->{'directory'} . $::settings->{'filename'})
			if $::widgets->{'directory_filename_entry'}->get_text ne $::settings->{'directory'} . $::settings->{'filename'};
		if ($widget eq $::widgets->{'directory_entry'}) {
			set_warning_text ($::widgets->{'directory_filename_entry'}, -w $value ? 0 : 1) ;
			message ("No permission to write to output directory!") if -w $value ? 0 : 1;
		}
	}
  if ($widget eq $::widgets->{'crop_width_spin'}) {
		my $wo = get_track()->{'width'};
		$::widgets->{'crop_horizontal_spin'}->set_adjustment(new Gtk2::Adjustment(0, 0, $wo - $value, 1, 50, 200));
		set_bpp();
	}
	if ($widget eq $::widgets->{'crop_horizontal_spin'}) {
		my $wo = get_track()->{'width'};
		$::widgets->{'crop_width_spin'}->set_adjustment(new Gtk2::Adjustment(0, 0, $wo - $value, 1, 50, 200));
		set_bpp();
	}
	if ($widget eq $::widgets->{'crop_height_spin'}) {
		my $wo = get_track()->{'height'};
		$::widgets->{'crop_vertical_spin'}->set_adjustment(new Gtk2::Adjustment(0, 0, $wo - $value, 1, 50, 200));
		set_bpp();
	}
	if ($widget eq $::widgets->{'crop_vertical_spin'}) {
		my $wo = get_track()->{'height'};
		$::widgets->{'crop_height_spin'}->set_adjustment(new Gtk2::Adjustment(0, 0, $wo - $value, 1, 50, 200));
		set_bpp();
	}
	if ($widget eq $::widgets->{'cache_directory_entry'}) {
		set_warning_text ($widget, -w $value ? 0 : 1)
	}
	if ($widget eq $::widgets->{'dvd_device_entry'}) {
		set_warning_text ($widget, -r $value ? 0 : 1)
	}
	1;
}

sub on_save_settings_button_clicked
{
	message("Settings saved");
	$::settings->save_settings;
}

sub on_restore_defaults_button_clicked
{
	$::settings->restore_settings;
	load_settings_to_interface();
}

sub progress_dialog_show
{
	if ($::widgets->{'cache_menc_vbox'}->parent == $::widgets->{'right_vbox'}) {
		$::widgets->{'acidrip'}->hide;
		$::widgets->{'right_vbox'}->remove($::widgets->{'cache_menc_vbox'});
		$::widgets->{'progress_vbox'}->add($::widgets->{'cache_menc_vbox'});
		
		$::widgets->{'status_hbox'}->remove($::widgets->{'status_bar'});
		$::widgets->{'progress_vbox'}->add($::widgets->{'status_bar'});
		
		$::widgets->{'progress_dialog'}->show;
	}
}

sub progress_dialog_hide
{
	if ($::widgets->{'cache_menc_vbox'}->parent == $::widgets->{'progress_vbox'}) {
		$::widgets->{'progress_dialog'}->hide;
		$::widgets->{'progress_vbox'}->remove($::widgets->{'cache_menc_vbox'});
		$::widgets->{'right_vbox'}->pack_start($::widgets->{'cache_menc_vbox'}, 0, 1, 0);

		$::widgets->{'progress_vbox'}->remove($::widgets->{'status_bar'});
		$::widgets->{'status_hbox'}->add($::widgets->{'status_bar'});
		
		$::widgets->{'acidrip'}->show;
	}
}

sub on_start_button_clicked
{
	my %menc = get_parameters();
	my @cache_files = ();
	
	if ( ! defined $::dvd->{'track'} ) {
		message ("Hey, you gotta load the DVD first!");
		return -1
	}
	
	while (Gtk2->events_pending()) { Gtk2->main_iteration() };

	if ( -e $menc{'output'} && ! $::settings->{'overwrite'}) {
		message("File already exists, refusing to overwrite");
		return -1;
	}
	if ( ! -d $::settings->{'directory'} ) {
		message("Output directory does not exist!");
		return -1;
	}
	if ( ! -w $::settings->{'directory'} ) {
		message("Can't write to output directory!");
		return -1;
	}
	if ( ( ! $::settings->{'enough_space'} ) && $::settings->{'enforce_space'}) {
		message("Not enough space in output directory!");
		return -1;
	}
	
	$::widgets->{'cache_menc_vbox'}->show;
	$::widgets->{'stop_button'}->show;
	$::widgets->{'start_button'}->hide;
	$::widgets->{'preview_button'}->set_sensitive(0);
	$::widgets->{'view_button'}->set_sensitive(0);
	$::widgets->{'progress_dialog_show_button'}->set_sensitive(1);	
	
	progress_dialog_show if $::settings->{'compact'};

	# cache if desired
	if ($::settings->{'cache'}) {
		CACHE:
		for (1..$::settings->{'total_blocks'}) {		
			$::settings->{'this_block'} = $_ if $::settings->{'total_blocks'} > 1;
			my $cache = get_command('cache');
			message("Running $cache");
			message("Caching title...");
			@cache_files = (@cache_files, cache ($cache));
			last CACHE if $?
		}
		$::settings->{'this_block'} = 0;
		if ($?) {
			$::widgets->{'stop_button'}->hide;
			$::widgets->{'start_button'}->show;
			message (($?>>8) == 1 ? "Mplayer caching interrupted by user" : "Mplayer caching exited with an error code " . ($? >> 8));
			return;
		}
		message ("Caching completed!");
		system "eject $::settings->{'dvd_device'}" if $::settings->{'eject'};
	}

	ENCODE:
	for (1..$::settings->{'total_blocks'}) {
		$::settings->{'this_block'} = $_ if $::settings->{'total_blocks'} > 1;
		my $menc;
		my $message = "Encoding film";
		$message .= ", Block " . $::settings->{'this_block'} . " of " . $::settings->{'total_blocks'} if $::settings->{'this_block'} > 0;

		if ($::settings->{'video_2pass'}) {

			$::settings->{'video_2pass'} = 1;
			$menc = get_command("mencoder");
			message("Running $menc");
			message($message .  ", 1st pass...");
			unlink "frameno.avi";
			encode($menc);
			last ENCODE if $?;

			$::settings->{'video_2pass'} = 2;
			$menc = get_command("mencoder");
			message("Running $menc\n");
			message($message . ", 2nd pass...");
			encode($menc) if !$?;
			last ENCODE if $?;
			unlink "divx2pass.log";

		} else { 
			$menc = get_command("mencoder");
			message("Running $menc");
			message($message . "...");
			encode($menc);
			last ENCODE if $?;
		}
	} system "eject $::settings->{'dvd_device'}" if $::settings->{'eject'} && ! $::settings->{'cache'};

	progress_dialog_hide if $::settings->{'compact'};

	if ($::settings->{'cache'} && $::settings->{'del_cache'}) {
		message("Deleting cache files");
		unlink @cache_files;
	}

	$::widgets->{'stop_button'}->hide;
	$::widgets->{'start_button'}->show;
	$::widgets->{'preview_button'}->set_sensitive(1);
  $::widgets->{'view_button'}->set_sensitive(1);
  $::widgets->{'progress_dialog_show_button'}->set_sensitive(0);

	message($? ? (($?>>8) == 1 ? "Mencoder interrupted by user" : "Mencoder exited with an error code " . ($? >> 8)) : "Encoding finished, hope it worked...");
	$::settings->{'this_block'} = 0;
	set_bitrate();
}

sub on_stop_button_clicked
{
	my $confirm = Gtk2::MessageDialog->new($::widgets->{'acidrip'},  [qw/modal destroy-with-parent/], 'question', 'yes-no', "Are you sure you want to stop the encoding process?");
	if ( $confirm->run eq 'yes' ) {
		kill_mplayer();
	}
	$confirm->destroy;
}

sub on_stop_preview_button_clicked
{
	kill_mplayer();
}

sub on_mencoder_output_show_button_clicked
{
	$::widgets->{'mencoder_output_frame'}->show_all;
	$::widgets->{'mencoder_output_show_button'}->hide;
}		
	
sub on_mencoder_output_hide_button_clicked
{
	$::widgets->{'mencoder_output_frame'}->hide;
	$::widgets->{'mencoder_output_show_button'}->show_all;
	$::widgets->{'status_hbox'}->show_all;
}	

sub on_mencoder_output_save_button_clicked
{
	if (open (LOGFILE, '>', "$ENV{HOME}/acidrip.log"))
	{
		print LOGFILE $::widgets->{'mencoder_output_text'}->get_buffer->get_text(
			$::widgets->{'mencoder_output_text'}->get_buffer->get_bounds, 0);
		close LOGFILE;
		message("Output log saved to $ENV{HOME}/acidrip.log")
	} else {
		message("Error opening output file")
	}
}

sub on_mencoder_output_saveall_button_clicked
{
	$::widgets->{'mencoder_output_save_button'}->set_sensitive(! $::widgets->{'mencoder_output_saveall_button'}->get_active);
	if ($::widgets->{'mencoder_output_saveall_button'}->get_active) {
		open (LOGFILE, '>', "$ENV{HOME}/acidrip.log");
		print LOGFILE $::widgets->{'mencoder_output_text'}->get_buffer->get_text(
			$::widgets->{'mencoder_output_text'}->get_buffer->get_bounds, 0);
	} else { close LOGFILE }
}

sub on_mencoder_output_clear_button_clicked
{
	message("Log information cleared");
	$::widgets->{'mencoder_output_text'}->get_buffer->delete(
		$::widgets->{'mencoder_output_text'}->get_buffer->get_bounds);
}

sub on_view_button_clicked
{
	#fork proofing
	open STDIN,  '/dev/null' or message("Can't read /dev/null: $!. May fail if in background");
	
	message("Playing back encoded track");
	my $mplayer_output;
	open (VIEW, get_command("view") . " 2>&1 |") || (message("mplayer preview failed, is it installed?"));
	while (<VIEW>) { 
		$mplayer_output .= $_; 
		while (Gtk2->events_pending()) {Gtk2->main_iteration()}
	}
	message($mplayer_output);
	message("Playing back encoded track... Finished");
}

sub on_read_dvd_button_clicked
{
  my ( $button, $widgets ) = @_;
  my $model;

	message("Loading DVD... please wait");
	$model = Gtk2::TreeStore->new ('Glib::String', 'Glib::String');
	$widgets->{'dvd_tree'}->set_model($model);

	while (Gtk2->events_pending()) {Gtk2->main_iteration()}

	my $ret = read_disc($::settings->{'lsdvd'}, $::settings->{'dvd_device'});

	message($::dvd->{'status'}, $ret ? "error" : "");
	if (defined $::dvd->{'track'})
  {
		my $selection = $widgets->{'dvd_tree'}->get_selection;

		$selection->set_mode('multiple');

    foreach my $this_track (@{$::dvd->{'track'}})
    {
			my $iter = $model->append(undef);
			my $ix = $this_track->{'ix'};
      my $chapters = @{$this_track->{'chapter'}};

			$model->set ($iter, 0, $ix, 1, $ix . ": " . hhmmss($this_track->{'length'}));

			foreach my $this_chapter (@{$this_track->{'chapter'}})
      {
				my $chap_iter = $model->append($iter);
				my $chap_ix = $this_chapter->{'ix'};

				$model->set ($chap_iter, 0, $chap_ix, 1, $chap_ix . ": " . hhmmss($this_chapter->{'length'}));
      }
    }
		my $cell = Gtk2::CellRendererText->new;

		my $column = Gtk2::TreeViewColumn->new_with_attributes ( "content", $cell, 'text', 1);
		
		$widgets->{'dvd_tree'}->remove_column($widgets->{'dvd_tree'}->get_column(0)) if (scalar $widgets->{'dvd_tree'}->get_columns());
		$widgets->{'dvd_tree'}->append_column($column);

		$selection->signal_connect (changed => \&on_select_track, $model);

		my $path = Gtk2::TreePath->new_from_string($::dvd->{'longest_track'} - 1);
		$selection->select_path($path);
		$widgets->{'dvd_tree'}->scroll_to_cell($path);

		$widgets->{'dvd_tree'}->show_all;

		my $name=$::dvd->{'title'};
		$name=~tr/A-Z/a-z/;	
		set_setting('filename', $name);

		$widgets->{'preview_button'}->set_sensitive(1);
		$widgets->{'start_button'}->set_sensitive(1);
	}
}	

1;
